@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import, division, print_function
 
 import pytest
 
@@ -23,21 +22,35 @@ class CustomError(exc.CryptoError):
     pass
 
 
+# Type safety: mypy can spot comparisons that will always evaluate to False, and the
+# bad argument type. Suppress these: we want to test these are detected at runtime.
+
+
 def test_exceptions_ensure_with_true_condition():
-    exc.ensure(1 == 1, 'one equals one')
+    exc.ensure(1 == 1, "one equals one")
 
 
 def test_exceptions_ensure_with_false_condition():
     with pytest.raises(exc.AssertionError):
-        exc.ensure(1 == 0, 'one is not zero',
-                   raising=exc.AssertionError)
+        exc.ensure(
+            1 == 0,  # type: ignore[comparison-overlap]
+            "one is not zero",
+            raising=exc.AssertionError,
+        )
 
 
 def test_exceptions_ensure_with_unwanted_kwarg():
     with pytest.raises(exc.TypeError):
-        exc.ensure(1 == 1, unexpected='unexpected')
+        exc.ensure(
+            1 == 1,
+            unexpected="unexpected",  # type: ignore[arg-type]
+        )
 
 
 def test_exceptions_ensure_custom_exception():
     with pytest.raises(CustomError):
-        exc.ensure(1 == 0, 'Raising a CustomError', raising=CustomError)
+        exc.ensure(
+            1 == 0,  # type: ignore[comparison-overlap]
+            "Raising a CustomError",
+            raising=CustomError,
+        )
